SystemOrganization addCategory: #'SWE-IssueTracker'!

TestCase subclass: #ITTestGlorpDB
	instanceVariableNames: 'login accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestGlorpDB methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 14:52'!
setUp
	
	super setUp.
	login := ITGlorpDB login.
	accessor := DatabaseAccessor forLogin: login.! !

!ITTestGlorpDB methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 15:28'!
testConnect
	
	| session |
	session := ITGlorpDB connect.
	
	self assert: (login username = session currentLogin username) description: 'Username of session does not equal login username'.
	self assert: (login connectString = session currentLogin connectString) description: 'ConnectString of session does not equal login ConnectString'.
	self assert: (login password = session currentLogin password) description: 'Password of session does not equal login password '.
	
	self assert: session isLoggedIn.! !

!ITTestGlorpDB methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 15:00'!
testLogin
	
	self assert: accessor isLoggedIn not.
	accessor loginIfError: [:exception | self assert: false.].
	self assert: accessor isLoggedIn.
	accessor logout.
	self assert: accessor isLoggedIn not.! !

!ITTestGlorpDB methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 19:38'!
testUniqueSession
	"Tests if a glorp session really is a singleton."

	| session |
	session := ITSession new.
	self assert: (session glorpSession = nil) not description: 'Glorp session is nil.'.
	self assert: (session glorpSession == session glorpSession) description: 'Glorp session is not unique.'.! !

TestCase subclass: #ITTestImageDB
	instanceVariableNames: 'database'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestImageDB methodsFor: 'running' stamp: 'afk 12/13/2007 18:47'!
setUp

	super setUp.
	database := ITImageDB connect.! !

!ITTestImageDB methodsFor: 'running' stamp: 'mig 12/4/2007 18:30'!
tearDown
	super tearDown! !

!ITTestImageDB methodsFor: 'testing' stamp: 'mig 12/4/2007 18:39'!
testAddComment
	| issue comment | 
	
	issue := ITIssue new.
	issue := database addIssue: issue.
	database addComment: 'TESTCOMMENT' atID: issue id.
	issue := database getIssueWithID: issue id.
	
	comment := issue comments first.
	
	self assert: (comment isKindOf: ITIssueComment).
	self assert: (comment content) = 'TESTCOMMENT'.
	
	database deleteIssueWithID: issue id.! !

!ITTestImageDB methodsFor: 'testing' stamp: 'mig 12/4/2007 18:20'!
testAddRemoveIssue

	| newIssue returningIssue |
	newIssue := ITIssue new.

	"add"
	returningIssue := database addIssue: newIssue.
	self assert: ( returningIssue isKindOf: ITIssue ) description: 'Returning Object is not kind of ITIssue'.
	self deny: ( returningIssue id isNil ) description: 'Returning ID is nil'.
	
	"remove"
	database deleteIssueWithID: newIssue id.
	self assert: (database getIssueWithID: newIssue id) isNil description: 'Issue was not deleted from database.'.! !

!ITTestImageDB methodsFor: 'testing' stamp: 'afk 12/13/2007 18:53'!
testGetIssue
	
	"I'm testing wheter ImageDB is only returning a copy of the object or not."
	| id issue | 
	
	issue := ITIssue new.
	issue := database addIssue: issue.
	database addComment: 'TESTCOMMENT' atID: issue id.
	database setDetailWith: 'title' to: 'testGetIssue Title' atID: issue id.

	id := database getIssues first id.
	
	self deny: ( (database getIssueWithID: id) == (database getIssueWithID: id) ) description: 'ITImageDB is not giving only a copy of the Object'.
	
	database deleteIssueWithID: issue id.! !

!ITTestImageDB methodsFor: 'testing' stamp: 'mig 12/4/2007 18:08'!
testGetIssues

	| issues |

	issues := (database getIssues).
	
	self assert: ( (issues intersection: (database getIssues)) isEmpty ) description: 'Is not returning only a copy of the objects'.
	self assert: ( (issues reject: [:issue | issue isKindOf: ITIssue]) isEmpty ) description: 'Returns objects which are not of kind ITIssue'! !

!ITTestImageDB methodsFor: 'testing' stamp: 'afk 12/13/2007 18:14'!
testSetTag
	| issue | 
	
	issue := ITIssue new.
	issue := database addIssue: issue.
	database setDetailWith: 'TESTTAG' to: 'TESTVALUE' atID: issue id.
	issue := database getIssueWithID: issue id.
	
	self assert: (issue getTag: 'TESTTAG') = 'TESTVALUE'.
	
	database deleteIssueWithID: issue id.! !

TestCase subclass: #ITTestIssue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

TestCase subclass: #ITTestIssueManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssueManager methodsFor: 'testing' stamp: 'afk 12/13/2007 18:54'!
testAddIssue
	
	| IssueManager returningIssue |
	IssueManager := ITIssueManager new.
	
	returningIssue := IssueManager addIssueWithTitle: 'A Test Issue'.
	self assert: (returningIssue isKindOf: ITIssue) description: 'addIssue is not returning an Issue object'.
	
	IssueManager deleteIssueWithID: returningIssue id.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'afk 12/13/2007 18:55'!
testDeleteIssue

	| IssueManager returningIssue issuelist |
	IssueManager := ITIssueManager new.
	
	returningIssue := IssueManager addIssueWithTitle: 'A Test Issue'.
	self assert: (returningIssue isKindOf: ITIssue) description: 'no issue present to delete'.
	IssueManager deleteIssueWithID: returningIssue id.
	
	issuelist := IssueManager getIssues select: [:issue | returningIssue id == issue id].
	self assert: (issuelist isEmpty).
	
	! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mfp 11/18/2007 17:23'!
testGetIssues
	
	| issueManager |
	issueManager := ITIssueManager new.
	
	issueManager getIssues do: [:issue|
		self assert: (issue isKindOf: ITIssue) description: 'not all returning objects are issues'].
	
	
	! !

WAComponent subclass: #ITComponentAddComment
	instanceVariableNames: 'comment issueID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAddComment class methodsFor: 'as yet unclassified' stamp: 'mig 11/30/2007 19:36'!
newWith: anID
	| newInstance |
	newInstance := self new.
	newInstance issueID: anID.
	^newInstance.! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:11'!
comment
	^ comment! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:11'!
comment: anObject
	comment := anObject! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:36'!
issueID
	^ issueID! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:36'!
issueID: anObject
	issueID := anObject! !

!ITComponentAddComment methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 20:35'!
renderContentOn: html
	html form: [
		html textArea on: #comment of: self.
		html break.
		html submitButton
			callback: [
				| issue issueManager |
				issueManager := self session issueManager.
				issue := issueManager getIssueWithID: self issueID.
				
				issueManager addComment: self comment atID: self issueID.
				
				"old ImageDB stuff TODO: Delete" 
				"issue addComment: self comment.
				issueManager saveChanges: issue."
			];
			text: 'Add Comment'.
	]! !

WAComponent subclass: #ITComponentAddIssue
	instanceVariableNames: 'title initialComment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
initialComment
	^ initialComment! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
initialComment: anObject
	initialComment := anObject! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mig 12/10/2007 17:48'!
renderContentOn: html
	html heading 
		level: 1;
		with: 'Add Issue'.
	html form: 
		[
		html text: 'Title'.
			html break.
		html textInput on: #title of: self.
			html break.
		html text: 'Description'.
			html break.
		html textArea on: #initialComment of: self.
			html break.
		html submitButton 
			callback: [
				self session issueManager addIssueWithTitle: self title withComment: self initialComment.
				self answer. 
			];
			value: 'add Issue'.
		html cancelButton
			callback: [self answer];
			value: 'cancel'.
		]! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
title
	^ title! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
title: anObject
	title := anObject! !

WAComponent subclass: #ITComponentDetails
	instanceVariableNames: 'issueID addcomment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentDetails class methodsFor: 'as yet unclassified' stamp: 'mig 11/30/2007 19:36'!
newWith: anID
	|view|
	view := self new.
	view issueID: anID.
	^view.! !

!ITComponentDetails methodsFor: 'as yet unclassified' stamp: 'mig 11/30/2007 19:16'!
children
	^ Array with: addcomment.! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:34'!
issueID
	^ issueID! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:34'!
issueID: anObject
	issueID := anObject! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mig 12/10/2007 17:49'!
renderContentOn: html

	| issue |
	
	issue := self session issueManager getIssueWithID: self issueID.
	addcomment := ITComponentAddComment newWith: self issueID.

	html heading
		level: 2;
		with: 'detailed view of an issue'.
	html div
		with: [
			html text: issue id.
			html break.
			html text: (issue getTag: 'title').
			
			html break; break.
			html table: [
				(issue comments) do: [:comment|
					html tableRow: [ html tableData: [ html text: comment content ] ].

				].
				html tableRow: [ 
					html tableData with: addcomment.
				].
			].
			html anchor
				callback: [self answer];
				with: [html text: 'return to list'].
		]
					
				! !

WAComponent subclass: #ITComponentListIssues
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentListIssues methodsFor: 'rendering' stamp: 'afk 12/13/2007 21:54'!
renderContentOn: html
	| issues |
	issues := self session issueManager getIssues.
	html heading 
		level: 2;
		with: 'List all issues'.
	html form:[
		html table:[
			html tableRow: [
				html tableData: 'id'.
				html tableData: 'title'.
				html tableData: 'details'.
				html tableData: 'delete'.
				].
			issues do: [:issue|
				html tableRow: [
					html tableData: (issue id asString).
					html tableData: (issue getTag: 'title').
					html tableData: [
						html anchor 
							callback: [self call: (ITComponentDetails newWith: issue id)];
							with: [html text: 'detailed view'].
						].
					html tableData: [
						html anchor
							callback: [((self session issueManager) deleteIssueWithID: issue id)];
							with: [html text: 'delete'].
						].
					].
				].
			].
		].! !

WAComponent subclass: #ITComponentMenu
	instanceVariableNames: 'listIssues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentMenu class methodsFor: 'as yet unclassified' stamp: 'mt 11/23/2007 18:52'!
canBeRoot
	^true.! !

!ITComponentMenu methodsFor: 'initialize' stamp: 'mt 11/23/2007 19:05'!
children
	^Array with: self listIssues.! !

!ITComponentMenu methodsFor: 'initialize' stamp: 'mt 11/23/2007 19:05'!
initialize
	super initialize.
	self listIssues: ITComponentListIssues new.! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mt 11/23/2007 19:05'!
listIssues
	^ listIssues! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mt 11/23/2007 19:05'!
listIssues: anObject
	listIssues := anObject! !

!ITComponentMenu methodsFor: 'rendering' stamp: 'mt 11/23/2007 19:51'!
renderContentOn: html
	html heading
		level: 1;
		with: 'IssueTracker'.
	html anchor 
		callback: [ self call: ITComponentAddIssue new ]; 
		with: [ html text: 'add issue'].
	html div
		with: self listIssues.! !

Object subclass: #GlorpStuff
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

Object subclass: #ITGlorpDB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITGlorpDB class methodsFor: 'connection' stamp: 'mt 12/14/2007 09:29'!
connect
	"Connects to the database with the given information from login and returns the current (new) session."	
	
	| accessor session system |
	accessor := DatabaseAccessor forLogin: self login.
	accessor login.

	session := ITGlorpSession new.
	session accessor: accessor.

	system := ITGlorpDescriptorSystem new.
	session system: system.
	system platform: session platform.
	
	^ session
! !

!ITGlorpDB class methodsFor: 'debug' stamp: 'mt 12/14/2007 09:59'!
createTables
	"I'm creating alle relevant tables but only if there are none of them."
	
	| session |
	session := ITGlorpDB connect.
	session inTransactionDo: [session system allTables do:
		[:each | session accessor createTable: each ifError: [:error | Transcript show: error messageText]]].! !

!ITGlorpDB class methodsFor: 'connection' stamp: 'afk 12/13/2007 15:01'!
login
		
	^ Login new database: PostgreSQLPlatform new;
		username: 'issuetracker';
		password: 'weloveseasidedontwe';
		connectString: 'theresnoplacelike.homeftp.org:5432_issuetracker'.! !

Object subclass: #ITImageDB
	instanceVariableNames: ''
	classVariableNames: 'ISSUES NEXTID'
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITImageDB class methodsFor: 'as yet unclassified' stamp: 'afk 11/12/2007 16:11'!
connect

	^ self new.! !

!ITImageDB methodsFor: 'Issues' stamp: 'mig 11/30/2007 18:29'!
addComment: aComment atID: id
	| issue |
	issue := 	(ISSUES at: id).
	issue comments add: (ITIssueComment newWithID: issue nextcommentid WithString: aComment).
	issue nextcommentid: issue nextcommentid+1.! !

!ITImageDB methodsFor: 'Issues' stamp: 'afk 11/30/2007 14:17'!
addIssue: anIssue

	anIssue id: NEXTID.
	ISSUES at: anIssue id put: anIssue.
	NEXTID := NEXTID+1.
	
	^ self getIssueWithID: anIssue id.! !

!ITImageDB methodsFor: 'initialize-release' stamp: 'mig 11/30/2007 18:50'!
clear

	ISSUES := Dictionary new.
	NEXTID := 0.
	! !

!ITImageDB methodsFor: 'Issues' stamp: 'afk 11/30/2007 14:30'!
deleteIssueWithID: id
	ISSUES removeKey: id.! !

!ITImageDB methodsFor: 'Issues' stamp: 'mig 12/4/2007 18:17'!
getIssueWithID: id

	"I'm returning a copy of the ITIssue gequested by the given id"
	^ (ISSUES at: id ifAbsent: nil) veryDeepCopy.
	! !

!ITImageDB methodsFor: 'Issues' stamp: 'mig 11/30/2007 18:30'!
getIssues
	^ ISSUES collect: [:anIssue | self getIssueWithID: anIssue id]
	! !

!ITImageDB methodsFor: 'initialize-release' stamp: 'afk 12/13/2007 18:25'!
initialize

	ISSUES ifNil: [ISSUES := Dictionary new].
	NEXTID ifNil: [NEXTID := 0].
	super initialize.! !

!ITImageDB methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 18:11'!
setDetailWith: aTag to: aValue atID: anID
	(ISSUES at: anID) details add: (ITIssueDetail with: aTag with: aValue).! !

Object subclass: #ITIssue
	instanceVariableNames: 'id details comments nextcommentid changeset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssue class methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 20:12'!
with: aTitle with: aComment

	| newIssue comment |
	newIssue := self new.
	comment := ITIssueComment new.
	
	comment content: aComment.
	
	newIssue comments add: comment.
	newIssue details add: (ITIssueDetail with: 'title' with: aTitle).

	^ newIssue! !

!ITIssue methodsFor: 'changing' stamp: 'afk 12/13/2007 20:52'!
addComment: aComment

	self comments add: (ITIssueComment with: aComment).
	"self changeset add: (Array with: #addComment:atID: with: aComment with: self id)."! !

!ITIssue methodsFor: 'accessing' stamp: 'mig 11/30/2007 18:34'!
changeset
	^ changeset! !

!ITIssue methodsFor: 'accessing' stamp: 'mig 11/30/2007 18:34'!
changeset: anObject
	changeset := anObject! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 11/23/2007 19:55'!
comments
	^comments.! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 12/11/2007 15:23'!
comments: aCollection
	comments := aCollection.! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 17:03'!
details
	^ details! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 17:03'!
details: anObject
	details := anObject! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 18:18'!
getTag: tag
	details do: 
		[:each | 
		each tag = tag
			ifTrue: [^ each content]
		].
	^nil.! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 16:20'!
id
	^ id! !

!ITIssue methodsFor: 'changing' stamp: 'afk 11/12/2007 16:20'!
id: anObject
	id := anObject! !

!ITIssue methodsFor: 'initialize-release' stamp: 'afk 12/13/2007 18:15'!
initialize
	changeset := OrderedCollection new.
	tags := Dictionary new.
	details := OrderedCollection new.
	comments := OrderedCollection new.
	changeset := OrderedCollection new.
	nextcommentid := 0.
	super initialize! !

!ITIssue methodsFor: 'accessing' stamp: 'mig 11/30/2007 18:29'!
nextcommentid
	^ nextcommentid! !

!ITIssue methodsFor: 'accessing' stamp: 'mig 11/30/2007 18:29'!
nextcommentid: anObject
	nextcommentid := anObject! !

!ITIssue methodsFor: 'changing' stamp: 'afk 12/13/2007 18:01'!
setDetailWith: aTag to: aValue
	self changeset add: (Array with: #setDetailWith:to:atID: with: aTag with: aValue with: self id)! !

Object subclass: #ITIssueComment
	instanceVariableNames: 'id content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueComment class methodsFor: 'as yet unclassified' stamp: 'mig 11/30/2007 18:22'!
newWithID: aNumber WithString: aString
	^ self new
		id: aNumber;
		content: aString;
		yourself.! !

!ITIssueComment class methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 20:52'!
with: aComment
	^ self new
		content: aComment;
		yourself.! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
content
	^ content! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
content: anObject
	content := anObject! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
id
	^ id! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
id: anObject
	id := anObject! !

!ITIssueComment methodsFor: 'initialize-release' stamp: 'mt 12/14/2007 10:02'!
initialize
	
	super initialize.
	
	content := ''.! !

Object subclass: #ITIssueDetail
	instanceVariableNames: 'id tag content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueDetail class methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 18:10'!
with: aTag with: aValue
	
	| detail |
	detail := ITIssueDetail new.
	detail tag: aTag.
	detail content: aValue.
	
	^ detail.! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:54'!
content
	^ content! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:54'!
content: anObject
	content := anObject! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
id
	^ id! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
id: anObject
	id := anObject! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
tag
	^ tag! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
tag: anObject
	tag := anObject! !

Object subclass: #ITIssueManager
	instanceVariableNames: 'database'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueManager methodsFor: 'accessing' stamp: 'afk 12/13/2007 20:55'!
addComment: aComment atID: anID

	| issue glorpSession |
	glorpSession := WACurrentSession value glorpSession.
	glorpSession inUnitOfWorkDo: 
		[
		issue := glorpSession readOneOf: ITIssue where: [:each | each id = anID].
		glorpSession register: issue.
		issue addComment: aComment.
		]
	

"	database addComment: aComment atID: id."! !

!ITIssueManager methodsFor: 'changing' stamp: 'afk 12/13/2007 18:39'!
addIssueWithTitle: aTitle

	| anIssue |
	
	anIssue := ITIssue new.
	
	anIssue := database addIssue: anIssue.
	
	anIssue setDetailWith: 'title' to: aTitle.
			
	self saveChanges: anIssue.
	
	^ anIssue.! !

!ITIssueManager methodsFor: 'changing' stamp: 'mt 12/14/2007 10:13'!
addIssueWithTitle: aTitle withComment: aComment
	
	| session issue |
	session := WACurrentSession value glorpSession.
	issue := ITIssue with: aTitle with: aComment.
	
	session inUnitOfWorkDo: 
		[
		session register: issue.
		].
	
	
	"| anIssue |
	anIssue := ITIssue new.
	anIssue := database addIssue: anIssue.
	anIssue setDetailWith: 'title' to: aTitle;
			addComment: aComment.		
	self saveChanges: anIssue.
	^ anIssue."! !

!ITIssueManager methodsFor: 'accessing' stamp: 'afk 12/13/2007 21:59'!
deleteIssueWithID: anID
	
	
	| session issue |
	session := WACurrentSession value glorpSession.
	
	session inUnitOfWorkDo: 
		[
		issue := session readOneOf: ITIssue where: [:each | each id = anID].
		session delete: issue.
		"session register: issue."
		].
	
	"database deleteIssueWithID: id."
	! !

!ITIssueManager methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 20:31'!
getIssueWithID: anID
	"Return the ITIssue where the id equals ID"

	| glorpSession |
	glorpSession := WACurrentSession value glorpSession.
	^ glorpSession readOneOf: ITIssue where: [:each | each id = anID].
	

"	^database getIssueWithID: anID."! !

!ITIssueManager methodsFor: 'accessing' stamp: 'afk 12/13/2007 20:31'!
getIssues
	"Return all Issues that are in the Database"

	| glorpSession |
	glorpSession := WACurrentSession value glorpSession.
	"Should return ALL ITIssues in the database"
	^ glorpSession readManyOf: ITIssue.! !

!ITIssueManager methodsFor: 'initialize-release' stamp: 'afk 11/12/2007 16:22'!
initialize
	
	database := ITImageDB connect.
	super initialize.! !

!ITIssueManager methodsFor: 'changing' stamp: 'mig 11/30/2007 18:44'!
saveChanges: anIssue

	anIssue changeset do: [ :change |
		database perform: (change first) withArguments: (change allButFirst).
	]
! !

Object subclass: #ITRole
	instanceVariableNames: 'name rights'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITRole methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:34'!
hasRight: aSymbol
	^ rights contains: aSymbol.! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:32'!
name1
	^ name! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:32'!
name: anObject
	name := anObject! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:32'!
rights
	^ rights! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:32'!
rights: anObject
	rights := anObject! !

Object subclass: #ITSecurityManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

Object subclass: #ITUser
	instanceVariableNames: 'username passwordHash roles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
passwordHash
	^ passwordHash! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
passwordHash: anObject
	passwordHash := anObject! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
roles
	^ roles! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
roles: anObject
	roles := anObject! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
username
	^ username! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
username: anObject
	username := anObject! !

WASession subclass: #ITSession
	instanceVariableNames: 'issueManager securityManager glorpSession user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITSession methodsFor: 'accessing' stamp: 'mt 12/14/2007 10:16'!
glorpSession
	"Returns the glorp session for this session and if there is none it created one"
	
	| sessions |
	sessions := ITGlorpSession allInstances.
	sessions	size = 0 ifTrue: [^ ITGlorpDB connect.]
			ifFalse: [^ sessions first].! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:45'!
issueManager
	issueManager ifNil: [
		issueManager := ITIssueManager new.
	].
	
	^issueManager.! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:46'!
securityManager
	securityManager ifNil: [
		securityManager := ITSecurityManager new.
	].
	
	^securityManager.! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:46'!
user
	^ user! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:46'!
user: anObject
	user := anObject! !

GlorpSession subclass: #ITGlorpSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

DescriptorSystem subclass: #ITGlorpDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITGlorpDescriptorSystem commentStamp: 'mt 12/14/2007 09:40' prior: 0!
I'm the descriptor system that should be used in every new ITGlorpSession. I'm responsible for all mappings between the IT-Classes and the relevant tables on the SQL server.

You should modify #allTableNames and #constructAllClasses whenever you want to add new objects of the domain space for usage here.

#tableNamed<TABLE_NAME>    ... descriptions about the table structure on the server
#descriptorFor<CLASS_NAME> ... defines mappings between classes and tables
#classModelFor<CLASS_NAME> ... defines types (classes) for symbols (attributes) used by mappings

The relationals schemes can be found in the comments for any #tableNamed<TABLE_NAME>. Primary keys are underlined, e.g. SCHEME(ATTRIBUTE_A, ATTRIBUTE_B)
                        -----------!

!ITGlorpDescriptorSystem methodsFor: 'accessing' stamp: 'mt 12/14/2007 09:34'!
allTableNames
	"Here are all tables listed which hold all persistent data."
	
	^ #('ISSUE' 'COMMENT' 'DETAIL')! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:54'!
classModelForITIssue: aClassModel
	"The unique ID can be up to 1.073.741.823 - should be enough.
	 -> SmallInteger maxVal
	 4-Byte-Integers (signed) are used in the database"
	
	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #comments collection: OrderedCollection  of: ITIssueComment.
	aClassModel newAttributeNamed: #details collection: OrderedCollection  of: ITIssueDetail.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:55'!
classModelForITIssueComment: aClassModel
	"See #classModelForITIssue for explanation of ID."
	
	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #content type: String.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:55'!
classModelForITIssueDetail: aClassModel
	"See #classModelForITIssue for explanation of ID."

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #tag type: String.
	aClassModel newAttributeNamed: #content type: String.! !

!ITGlorpDescriptorSystem methodsFor: 'accessing' stamp: 'mt 12/14/2007 09:35'!
constructAllClasses
	"Here are all classes listed which are necessary for the mappings."
	
	^ (super constructAllClasses)
		add: ITIssue;
		add: ITIssueComment;
		add: ITIssueDetail;
		yourself! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 12/14/2007 09:45'!
descriptorForITIssue: aDescriptor
	"Every issue contains a unique ID, a collection of comments and a collection of details."
	
	| issueTable commentTable detailTable |
	issueTable   := self tableNamed: 'ISSUE'.
	commentTable := self tableNamed: 'COMMENT'.
	detailTable  := self tableNamed: 'DETAIL'.

	aDescriptor table: issueTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (issueTable fieldNamed: 'ID').

	(aDescriptor newMapping: ToManyMapping)
		attributeName: #comments;
		join: (Join
			from: (issueTable fieldNamed: 'ID')
			to: (commentTable	fieldNamed: 'ISSUE_ID')
		).
	
	(aDescriptor newMapping: ToManyMapping)
		attributeName: #details;
		join: (Join
			from: (issueTable fieldNamed: 'ID')
			to:  (detailTable fieldNamed: 'ISSUE_ID')
		).! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 12/14/2007 09:47'!
descriptorForITIssueComment: aDescriptor
	"Every comment contains a unique ID and its content - should be text."
	
	| table |
	table := self tableNamed: 'COMMENT'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #content to: (table fieldNamed: 'CONTENT').! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 12/14/2007 09:47'!
descriptorForITIssueDetail: aDescriptor
	"Every detail contains a unique ID, a tag and its content."

	| table |
	table := self tableNamed: 'DETAIL'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #tag to: (table fieldNamed: 'TAG').
	(aDescriptor newMapping: DirectMapping) from: #content to: (table fieldNamed: 'CONTENT').! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/14/2007 09:59'!
tableForCOMMENT: aTable
	"COMMENTS(ID, ISSUE_ID, TIMESTAMP, CONTENT)
	          --"
	
	| issueId |
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.
	issueId := aTable createFieldNamed: 'ISSUE_ID' type: (platform int4).
	aTable createFieldNamed: 'TIMESTAMP' type: (platform timestamp).
	aTable createFieldNamed: 'CONTENT' type: (platform text).
	
	"foreign key definition for the IDs of issues"
	aTable addForeignKeyFrom: issueId 
		to: ((self tableNamed: 'ISSUE') fieldNamed: 'ID').
		! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/14/2007 09:38'!
tableForDETAIL: aTable
	"DETAIL(ID, ISSUE_ID, TAG, CONTENT)
	       --"
	
	| issueID |
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.
	issueID := (aTable createFieldNamed: 'ISSUE_ID' type: (platform int4)).
	aTable createFieldNamed: 'TAG' type: (platform varchar: 255).
	aTable createFieldNamed: 'CONTENT' type: (platform varchar: 255).
	
	"foreign key definition for IDs of issues"
	aTable addForeignKeyFrom: issueID to: ((self tableNamed: 'ISSUE') fieldNamed: 'ID').! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/14/2007 09:38'!
tableForISSUE: aTable
	"ISSUE(ID)
	       --"
	
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.! !
